#include "global_define.h"
#include "define.h"
#include "MDIWindow.h"

#include <qmainwindow.h>
#include <qworkspace.h>


class QPopupMenu;
class QToolBar;
class QToolButton;
class QProgressBar;
class QStringList;


// extern CMainWindow *g_pMainWindow;


enum
{ 
	NONE_VIEW = 0,
	PLAN_VIEW,
	OPENGL_VIEW,
	RENDER_VIEW,
	MONTAGE_VIEW,
	MCUBE_VIEW
};

class CMainWindow : public QMainWindow
{
	Q_OBJECT

public:
	CMainWindow( QWidget* parent = 0, const char* name = 0, WFlags f = WType_TopLevel );
	~CMainWindow();

	void init();
	QMainWindow *newDoc( int type = PLAN_VIEW );
	void closeMDIWindow( QString str );

	void changeMenuAndToolBar( int type = NONE_VIEW );

	void checkOutRecentFile( QString );	// check out recent file

protected:
	void setMenuBar();
	void setToolBar();
	void setStatusBar();

private slots:
	// file menu slot
	void slotOpen();
	void slotSave();
	void slotImportStack();

	// view menu slot
	//      for MDIWindow
	void slotZoomIn();
	void slotZoomOut();
	void slotZoomAll();
	void slotXView();
	void slotYView();
	void slotZView();
	void slotNextSlice();
	void slotPreviousSlice();
	void slotVoxelInfo();
	//      for GLMDIWindow
//	void slotZoomAll();			// dup with MDIWindow
	void slotDefaultView();
//	void slotVoxelInfo();		// dup with MDIWindow
	void slotVolumeInfo();
	//		for RenderMDIWindow
//	void slotVoxelInfo();		// dup with MDIWindow

	// 3d model menu slot
	//    for MDIWindow
	void slotShowSegmentation();
	void slotSegmentation();
	void slotROI();
	void slotROIClearROI();
	void slotFEModel();
	void slotRenderVolume();
	void slotMontageView();
	void slotMarchingCubeView();
	//    for GLMDIWindow
	void slotMeshFiltering();
	void slotMeshFilterInfo();
	void slotAnalysisVolume();

	// image menu slot
	//    for MDIWIndow
	void slotResampling();
	//    for GLMDIWindow
	void slotSaveAsImage();
	//    for RenderMDIWindow
	void slotRenderedImageSize();
	void slotRenderInfo();

	// help menu slot
	void slotAboutMGIBA();

	// plan view toolbar slot
	void slotMeasureLength();
	void slotMeasureAngle();
//	void slotMeasureRegion();
	void slotShowMeasureLine();
	void slotMeasureClearLine();
	void slotFillSegmentation();
	void slotEraseSegmentation();
	void slotUpdate3DModel();

	// view toolbar slot
	void slotPlusX();
	void slotMinusX();
	void slotPlusY();
	void slotMinusY();
	void slotPlusZ();
	void slotMinusZ();
//	void slotZoomIn();		// dup with view menu slot
//	void slotZoomOut();		// dup with view menu slot

	// menu bar
	void slotFileMenuAboutToShow();
	void slotViewMenuAboutToShow();
	void slot3DModelMenuAboutToShow();
	void slotImageMenuAboutToShow();
	void slotHelpMenuAboutToShow();

	// popup
	void slotExportPopupAboutToShow();
	void slotRecentFilesPopupAboutToShow();
	void slotToolPopupAboutToShow();
	void slotImagePalettePopupAboutToShow();
	void slotPreDefinedViewPopupAboutToShow();

	// export popup
	void slotExportToAnsys();
	void slotExportToSTLAscii();
	void slotExportToAcadScript();
	void slotExportToAcadDxf();

	// tool popup
	void slotToolMainToolBar();
	void slotToolSegMeasureToolBar();
	void slotToolViewToolBar();
	void slotToolFEModelToolBar();

	// image palette popup
	void slotImagePaletteWhiteToBlackColor();
	void slotImagePaletteBlackToWhiteColor();
	void slotImagePaletteSingleColor();
	void slotImagePaletteIndexColor();
	void slotImagePaletteSelectSingleColor();
	void slotImagePaletteSelectIndexRange();

	void slotFilteredShowHide();
	void slotTitleShowHide();
	void slotBoundingBoxShowHide();
	void slotGridLineShowHide();

	// etc
	void slotWindowActivated( QWidget *w );

	// recent files
	void slotRecentFile1();
	void slotRecentFile2();
	void slotRecentFile3();
	void slotRecentFile4();

private:
	void recentFileOpen( QString );		// open recent file
	void checkInRecentFile( QString );	// check in recent file
	bool checkVolumeList( QString );	// check volume is in the workspace or not

protected:
	virtual void closeEvent( QCloseEvent * );

protected:
	QWorkspace *ws;

public:
	QPopupMenu *m_pFileMenu;
	QPopupMenu *m_pViewMenu;
	QPopupMenu *m_p3DModelMenu;
	QPopupMenu *m_pImageMenu;
	QPopupMenu *m_pHelpMenu;

	int m_nFileMenuId;
	int m_nViewMenuId;
	int m_n3DModelMenuId;
	int m_nImageMenuId;
	int m_nHelpMenuId;

	QPopupMenu *m_pExportPopup;
	QPopupMenu *m_pRecentFilesPopup;
	QPopupMenu *m_pToolPopup;
	QPopupMenu *m_pImagePalettePopup;
	QPopupMenu *m_pPreDefinedViewPopup;

	QToolBar *m_pMainToolBar;
	QToolBar *m_pPlanViewToolBar;	// segmentation, roi & measure...  for plan view
	QToolBar *m_pViewToolBar;
	QToolBar *m_pFEModelToolBar;	// for opengl view

public:
	QToolButton *m_pSegmentationShowHide;
	QToolButton *m_pSegmentation;

	QToolButton *m_pROI;
	QToolButton *m_pROIShowHide;

	QToolButton *m_pMeasureLength;
	QToolButton *m_pMeasureAngle;
//	QToolButton *m_pMeasureRegion;
	QToolButton *m_pMeasureLineShowHide;
	QToolButton *m_pFillSegmentation;
	QToolButton *m_pEraseSegmentation;

	QToolButton *m_pZoomIn;
	QToolButton *m_pZoomOut;

	QToolButton *m_pWhiteToBlackColor;
	QToolButton *m_pBlackToWhiteColor;
	QToolButton *m_pSingleColor;
	QToolButton *m_pIndexColor;
	QToolButton *m_pFilteredShowHide;
	QToolButton *m_pTitleShowHide;
	QToolButton *m_pBoundingBoxShowHide;
	QToolButton *m_pGridLineShowHide;

	// recent files
	int m_nRecentFiles;
	QString m_strRecentFiles[4];

	// volume list
	QStringList m_listVolume;
};