#include "MarchingCubeMDIWindow.h"
#include "MarchingCubeView.h"
#include "MDIWindow.h"
#include "VolumeFile.h"
#include "SimpleMessageDlg.h"

#include <qlabel.h>

#include <vtkUnsignedCharArray.h>
#include <vtkImageData.h>
#include <vtkMarchingCubes.h>
#include <vtkPolyDataMapper.h>
#include <vtkActor.h>
#include <vtkOutlineFilter.h>
#include <vtkCamera.h>
#include <vtkSTLWriter.h>

MarchingCubeMDIWindow::MarchingCubeMDIWindow( QWidget *parent, const char *name, WFlags f )
	: QMainWindow( parent, name, f )
{
	m_pVolume = NULL;
	m_pMDIWindow = NULL;


	m_pVtkImageData = NULL;
	m_pVtkMarchingCubes = NULL;
	m_pVtkSkinPolyDataMapper = NULL;
	m_pVtkOutlinePolyDataMapper = NULL;
	m_pVtkOutlineFilter = NULL;
	m_pVtkSkinActor = NULL;
	m_pVtkOutlineActor = NULL;


	// MarchingCube view
	m_pMarchingCubeView = new CMarchingCubeView( this );
	ASSERT( m_pMarchingCubeView );

	setFocusProxy( m_pMarchingCubeView );
	setCentralWidget( m_pMarchingCubeView );
} // MarchingCubeMDIWindow

MarchingCubeMDIWindow::~MarchingCubeMDIWindow()
{
	if ( m_pMDIWindow )
		m_pMDIWindow->unsetMarchingCubeView();

	if ( m_pVtkImageData )
	{
		m_pVtkImageData->Delete();
		m_pVtkImageData = NULL;
	}
	if ( m_pVtkMarchingCubes )
	{
		m_pVtkMarchingCubes->Delete();
		m_pVtkMarchingCubes = NULL;
	}
	if ( m_pVtkSkinPolyDataMapper )
	{
		m_pVtkSkinPolyDataMapper->Delete();
		m_pVtkSkinPolyDataMapper = NULL;
	}
	if ( m_pVtkOutlinePolyDataMapper )
	{
		m_pVtkOutlinePolyDataMapper->Delete();
		m_pVtkOutlinePolyDataMapper = NULL;
	}
	if ( m_pVtkOutlineFilter )
	{
		m_pVtkOutlineFilter->Delete();
		m_pVtkOutlineFilter = NULL;
	}
	if ( m_pVtkSkinActor )
	{
		m_pVtkSkinActor->Delete();
		m_pVtkSkinActor = NULL;
	}
	if ( m_pVtkOutlineActor )
	{
		m_pVtkOutlineActor->Delete();
		m_pVtkOutlineActor = NULL;
	}
	if ( m_pMarchingCubeView )
	{
		delete m_pMarchingCubeView;
		m_pMarchingCubeView = NULL;
	}
} // ~MarchingCubeMDIWindow

bool MarchingCubeMDIWindow::init( CVolume *pVolume )
{
	m_pVolume = pVolume;
	ASSERT( m_pVolume );

	CSimpleMessageDlg *message = new CSimpleMessageDlg( this, "", false );
	message->setCaption( tr( "Surface Model" ) );
	message->IDC_LBL_MESSAGE->setText(
		tr( 
"Extracting Iso Surface\n\n"
"Please wait a minute..."
		 ) );
//	message->exec();
	message->show();

	int size[3];
	size[0] = m_pVolume->volumeWidth();
	size[1] = m_pVolume->volumeHeight();
	size[2] = m_pVolume->volumeDepth()+2;

	// make iso volume
	BYTE *pIsoVolume = NULL;
	pIsoVolume = new BYTE[ size[0]*size[1]*size[2] ];
	if ( !pIsoVolume )
	{
		return false;
	}

	// assign min & max layer to close iso surface
	DWORD i;
	for ( i=0 ; i<(DWORD)size[0]*size[1] ; i++ )
	{
		pIsoVolume[i] = 0;
		pIsoVolume[ size[0]*size[1]*(size[2]-1)+i ] = 0;
	}

	for ( i=0 ; i<(DWORD)size[0]*size[1]*(size[2]-2) ; i++ )
	{
		if ( m_pVolume->isMaskOn( i ) )
			pIsoVolume[ i+size[0]*size[1] ] = 127;
		else
			pIsoVolume[ i+size[0]*size[1] ] = 0;
	}

	// make iso surface
	vtkUnsignedCharArray *array = vtkUnsignedCharArray::New();
	array->SetVoidArray( pIsoVolume, size[0]*size[1]*size[2], 1 );

	m_pVtkImageData = vtkImageData::New();
	m_pVtkImageData->GetPointData()->SetScalars( array );
	m_pVtkImageData->SetDimensions( size );
	m_pVtkImageData->SetScalarType( VTK_UNSIGNED_CHAR );
	m_pVtkImageData->SetSpacing( m_pVolume->voxelWidth(),
						   m_pVolume->voxelHeight(),
						   m_pVolume->voxelDepth() );
	m_pVtkImageData->SetOrigin( 0.0, 0.0, 0.0 );
	array->Delete();
//	delete pIsoVolume;
//	pIsoVolume = NULL;

	//extract the skin
	m_pVtkMarchingCubes = vtkMarchingCubes::New();
	m_pVtkMarchingCubes->SetInput( m_pVtkImageData );
	m_pVtkMarchingCubes->SetValue( 0, 127 );

	m_pVtkSkinPolyDataMapper= vtkPolyDataMapper::New();
	m_pVtkSkinPolyDataMapper->SetInput( m_pVtkMarchingCubes->GetOutput() );
	m_pVtkSkinPolyDataMapper->ScalarVisibilityOff();

	m_pVtkSkinActor = vtkActor::New();
	m_pVtkSkinActor->SetMapper( m_pVtkSkinPolyDataMapper );

	//get an outline
	m_pVtkOutlineFilter = vtkOutlineFilter::New();
	m_pVtkOutlineFilter->SetInput( m_pVtkImageData );
	
	vtkPolyDataMapper *m_pVtkOutlinePolyDataMapper; 
	m_pVtkOutlinePolyDataMapper = vtkPolyDataMapper::New();
	m_pVtkOutlinePolyDataMapper->SetInput( m_pVtkOutlineFilter->GetOutput() );
	
	m_pVtkOutlineActor = vtkActor::New();
	m_pVtkOutlineActor->SetMapper( m_pVtkOutlinePolyDataMapper );
	m_pVtkOutlineActor->GetProperty()->SetColor( 0, 1, 0 );
	
	/*
	//create a camera with the correct view up
	vtkCamera *aCamera = vtkCamera::New();
	aCamera->SetViewUp( 0, 0, -1 );
	aCamera->SetPosition( 0, 1, 0 );
	aCamera->SetFocalPoint( 0, 0, 0 );
	aCamera->ComputeViewPlaneNormal();
	*/
	
	//tell the renderer our camera and actors
	m_pMarchingCubeView->getRenderer()->AddActor( m_pVtkOutlineActor );
	m_pMarchingCubeView->getRenderer()->AddActor( m_pVtkSkinActor );
//	m_pMarchingCubeView->getRenderer()->SetActiveCamera( aCamera );
	m_pMarchingCubeView->getRenderer()->ResetCamera();
	
	// set MDI windows' name
	QString fileName = m_pVolume->volumeFileName();
	if ( fileName.isEmpty() )
		fileName = "unnamed Surface	Model";
		
	setCaption( tr( "Surface Model : " ) + fileName );

	// resize
	resize( 512, 512 );

	if ( message )
	{
		message->hide();
		delete message;
		message = NULL;
	}

	return true;
} // init

CVolume *MarchingCubeMDIWindow::volume()
{
	if ( m_pVolume )
		return m_pVolume;
	else
		return NULL;
} // volume
	
CMarchingCubeView *MarchingCubeMDIWindow::marchingCubeView()
{
	if ( m_pMarchingCubeView )
		return m_pMarchingCubeView;
	else
		return NULL;
} // marchingCubeView

void MarchingCubeMDIWindow::setMDIWindow( MDIWindow *pMDIWindow )
{
	m_pMDIWindow = pMDIWindow;
} // setMDIWindow

MDIWindow *MarchingCubeMDIWindow::mdiWindow()
{
	if ( m_pMDIWindow )
		return m_pMDIWindow;
	else
		return NULL;
} // mdiWindow

bool MarchingCubeMDIWindow::exportToSTL( QString fn )
{
	CSimpleMessageDlg *message = new CSimpleMessageDlg( this, "", false );
	message->setCaption( tr( "Surface Model" ) );
	message->IDC_LBL_MESSAGE->setText( 
		tr( 
"Writing Iso Surface to STL\n\n"
"Please wait a minute..."
		 ) );
//	message->exec();
	message->show();

	vtkSTLWriter *writer = vtkSTLWriter::New();
	writer->SetInput( m_pVtkMarchingCubes->GetOutput() );
	writer->SetFileName( fn.latin1() );
	writer->Write();
	writer->Delete();

	if ( message )
	{
		message->hide();
		delete message;
		message = NULL;
	}

	return true;
} // exportToSTL