#include "MathExpParser.h"

#include <math.h>

#include <qstring.h>
#include <qstringlist.h>
#include <qregexp.h>

#include <qmessagebox.h>	//XXX
#include <iostream.h>		//XXX


////////////////////////////////////////////////////////////////////////////
//
// CMyVector
//
int CMyVector::compareItems( QCollection::Item item1, QCollection::Item item2 )
{
	QString *p1, *p2;

	p1 = (QString *) item1;
	p2 = (QString *) item2;

	if ( QString::compare( *p1, *p2 ) == 0 )
		return 0;
	else
		return 1;
}


////////////////////////////////////////////////////////////////////////////
//
// CMathExpParser
//
CMathExpParser::CMathExpParser( /*QWidget *parent*/ )
{
	/*
	m_pParent = parent;
	ASSERT( m_pParent );
	*/

	// initialize
	initFuncVectorName();
	initFuncVector();
} // CMathExpParser

CMathExpParser::~CMathExpParser()
{
	m_VectorFunc.clear();
	m_VectorFuncName.clear();
} // ~CMathExpParser

bool CMathExpParser::parse( QString strExp, double *result, int *error )
{
	strExp = strExp.lower();
	strExp = strExp.replace( QRegExp( " "), "" ); 

	unsigned int expLength = strExp.length();	// length of expression

	unsigned int jjCount = 0;
	for ( unsigned int jj=0 ; jj<expLength ; jj++ )
	{
		QChar c = strExp.at( jj );

		if ( c.isNumber() )
			jjCount++;
	}

	// if all characters are number, just convert and return true;
	if ( jjCount == expLength )
	{
		*result = strExp.toDouble();
		return true;
	}

	bool flag = true;
	int nCase;						// current case
	int nPreCase = 0;				// previous case
	unsigned int nPosition = 0;		// current index of QString
	QString tmpNum;
	int tmpCount = 0;
	QString tmpNum2;
	QStringList listNum;			// 1+1+2 => 1, 1, 2 will be stored.
	QChar listOp[256];				// 1+1*5 => +, * will be stored.
	short int opCount = 0;			// number of operator : 1+1*5 => opCount = 2
	
	while ( flag )
	{
		QChar c = strExp.at( nPosition );

		// check character
		if ( c.isNumber() )
			nCase = 1;
		else if ( c == '+' || c == '-' )
			nCase = 2;
		else if ( c == '*' || c == '/' || c == '^' )
			nCase = 3;
		else if ( c == '(' )
			nCase = 4;
		else if ( c == ')' )
			nCase = 5;
		else if ( c == '.' )
			nCase = 6;
		else if ( c.isLetter() )
			nCase = 7;
		else
			nCase = 8;

		// do parsing
		switch ( nCase )
		{
		case 1:		// number
			{
				tmpNum[tmpCount++] = c;
				if ( nPosition == expLength-1 )
				{
					QString *pStr = new QString( tmpNum );
					listNum.append ( *pStr );

					tmpNum = "";
					tmpCount = 0;
				}
			}
			break;
		case 2:		// +, -
			{
				// ex) 1+1+
				if ( nPosition == expLength-1 )
				{
					*error = expLength;
					return false;
				}

				// ex) 1++1, 1+-1, 1--1
				if ( nPreCase == 2 )
				{
					*error = nPosition;
					return false;
				}

				// ex) 1*+1, 1*-1, 1/+1, 1/-1
				if ( nPreCase == 3 && isOperator( strExp.at( nPosition-1 ) ) )
				{
					*error = nPosition;
					return false;
				}

				// ex) -1, +1...
				if ( nPosition == 0 )
				{
					QString *pStr = new QString( "0" );
					listNum.append( *pStr );
				}

				// ex) 1+1 => 1+,  (1+1)+1
				if ( ( nPreCase == 1 && tmpCount != 0 ) || nPreCase == 4 )
				{
					QString *pStr = new QString;
					*pStr = tmpNum;
					listNum.append( *pStr );

					tmpNum = "";
					tmpCount = 0;
				}
				
				listOp[opCount++] = c;
			}
			break;
		case 3:		// *, /, ^
			{
				// ex) 1+1*
				if ( nPosition == expLength-1 )
				{
					*error = expLength;
					return false;
				}

				// ex) 1-^1, 1+*1, 1/+1
				if ( nPreCase == 2 )
				{
					*error = nPosition+1;
					return false;
				}

				// ex ) 1**1, 1*/1
				if ( nPreCase == 3 && isOperator( strExp.at( nPosition-1 ) ) )
				{
					*error = nPosition+1;
					return false;
				}

				// ex) *1, /1, ^1
				if ( nPosition == 0 )
				{
					*error = nPosition+1;
					return false;
				}

				bool bFind = false;
				int ret;

				if ( strExp.at( nPosition+1 ) == '(' )
				{
					bool iflag = true;
					unsigned int j=2;
					int pCount = 0;

					while ( iflag )
					{
						QChar c = strExp.at( nPosition+j );

						if ( c == '(' )
							pCount++;
						else if ( c == ')' )
						{
							if ( pCount == 0 )
							{
								ret = nPosition+j;
								iflag = false;
								bFind = true;
							}
							else
								pCount--;
						}
						
						if ( nPosition+j < expLength )
							j++;
						else
							iflag = false;
					}

					if ( bFind )
					{
						int kk = 0;
						double val;
						int err;
						QString str;
						for ( int ii=nPosition+2 ; ii<ret ; ii++ )
							str[kk++] = strExp[ii];

						if ( parse( str, &val, &err ) )	// recursive call to parse
						{
							double d1;
							if ( nPreCase == 3 || nPreCase == 7 )
							{
								d1 = tmpNum2.toDouble();
								listNum.remove( listNum.at( listNum.count()-1 ) );
							}
							else
								d1 = tmpNum.toDouble();

							double rst;

							if ( c == '*' )
								rst = d1 * val;
							else if ( c == '/' )
								rst = d1 / val;
							else if ( c == '^' )
								rst = pow( d1, val );

							QString *pStr = new QString;
							pStr->setNum( rst );
							listNum.append( *pStr );

							tmpNum = "";
							tmpCount = 0;

							tmpNum2.setNum( rst );
						}
						else
						{
							*error = nPosition + err + 1;
							return false;
						}

						nPosition = ret;
					}
					else
					{
						*error = expLength;
						return false;
					}
				}
				else	// not found '('
				{
					unsigned int j=1;
					bool iflag = true;

					while ( iflag )
					{
						QChar c = strExp.at( nPosition+j );

						if ( isOperator( c ) )
						{
							ret = nPosition+j-1;
							iflag = false;
							bFind = true;
						}
						if ( !bFind && nPosition+j == expLength-1 )
						{
							ret = expLength-1;
							bFind = true;
						}

						if ( !bFind && (nPosition+j) < expLength )
							j++;
						else
							iflag = false;
					}	// while

					if ( bFind )
					{
						double val;
						int err;
						int kk = 0;
						QString str;
						for ( int ii=nPosition+1 ; ii<=ret ; ii++ )
							str[kk++] = strExp[ii];

						if ( parse( str, &val, &err ) )
						{
							double d1;
							if ( nPreCase == 3 || nPreCase == 7 )
							{
								d1 = tmpNum2.toDouble();
								listNum.remove( listNum.at( listNum.count()-1 ) );
							}
							else
								d1 = tmpNum.toDouble();

							double rst;

							if ( c == '*' )
								rst = d1 * val;
							else if ( c == '/' )
								rst = d1 / val;
							else if ( c == '^' )
								rst = pow( d1, val );

							QString *pStr = new QString;
							pStr->setNum( rst );
							listNum.append( *pStr );

							tmpNum = "";	// tmpNum.setNum( rst );
							tmpCount = 0;	// tmpNum.length();

							tmpNum2.setNum( rst );
						}
						else
						{
							*error = nPosition + err + 1;
							return false;
						}

						nPosition = ret;
					}	// if ( bFind )
					else
					{
						*error = expLength;
						return false;
					}
				}
			}
			break;
		case 4:		// (
			{
				// ex) 1(, 1+1(
				if ( !isOperator( strExp.at( nPosition-1 ) ) )
				{
					*error = nPosition+1;
					return false;
				}

				// ex) 1+1+(
				if ( nPosition == expLength-1 )
				{
					*error = expLength;
					return false;
				}

				bool iflag = true;
				unsigned int j=1;
				int pCount = 0;

				bool bFind = false;
				int ret;

				while ( iflag )
				{
					QChar c = strExp.at( nPosition+j );

					if ( c == '(' )
						pCount++;
					else if ( c == ')' )
					{
						if ( pCount == 0 )
						{
							ret = nPosition+j-1;
							iflag = false;
							bFind = true;
						}
						else
							pCount--;
					}
					
					if ( nPosition+j < expLength )
						j++;
					else
						iflag = false;
				}

				if ( bFind )
				{
					double val;
					int err;
					int kk = 0;
					QString str;
					for ( int ii=nPosition+1 ; ii<=ret ; ii++ )
						str[kk++] = strExp[ii];

					if ( str.length() == 0 )
					{
						*error = nPosition+1;
						return false;
					}

					if ( parse( str, &val, &err ) )
					{
						tmpNum.setNum( val );
						tmpCount = tmpNum.length();
					}
					else
					{
						*error = nPosition + err + 1;
						return false;
					}

					nPosition = ret+1;
				}
				else
				{
					*error = expLength;
					return false;
				}
			}
			break;
		case 5:		// )
			/*
			if ( nPosition == expLength-1 )
			{
				QString *pStr = new QString( tmpNum );
				listNum.append ( *pStr );

				tmpNum = "";
				tmpCount = 0;
			}
			else
			*/
			{
				*error = nPosition+1;
				return false;
			}
			break;
		case 6:		// .
			{
				if ( nPreCase != 6 )
					tmpNum[tmpCount++] = c;
				else
				{
					*error = nPosition+1;
					return false;
				}
			}
			break;
		case 7:		// letter
			{
				QString strFunc;
				bool iflag = true;
				unsigned int j=0;

				bool bFind = false;
				int ret;

				// find function name
				while ( iflag )
				{
					QChar c = strExp.at( nPosition+j );

					if ( c == '(' )
					{
						ret = nPosition+j+1;
						bFind = true;
						iflag = false;
					}
					else
					{
						strFunc += c;
					}
					
					if ( nPosition+j < expLength-1 )
						j++;
					else
						iflag = false;
				}

				if ( !bFind )
				{
					*error = nPosition+1;
					return false;
				}

				// check if the function is exist or not
				int funcIndex = m_VectorFuncName.find( &strFunc );
				if ( funcIndex == -1 )
				{
					*error = nPosition+1;
					return false;
				}

				// find parameter
				int pCount = 0;
				int ret2;

				bFind = false;
				iflag = true;
				j = 0;
				nPosition = ret;
				while ( iflag )
				{
					QChar c = strExp.at( nPosition+j );

					if ( c == '(' )
						pCount++;
					else if ( c == ')' )
					{
						if ( pCount == 0 )
						{
							ret2 = nPosition+j;
							iflag = false;
							bFind = true;
						}
						else
							pCount--;
					}
					
					if ( nPosition+j < expLength )
						j++;
					else
						iflag = false;
				}

				if ( bFind )
				{
					double val;
					int err;
					int kk = 0;
					QString strPara;
					for ( int ii=ret ; ii<ret2 ; ii++ )
						strPara[kk++] = strExp[ii];

					if ( strPara.length() == 0 )
					{
						*error = ret;
						return false;
					}

					if ( parse( strPara, &val, &err ) )
					{
						MATH_EXP_FUNC *mef = m_VectorFunc[funcIndex];

						double rst = (mef->func) ( val );

						QString *pStr = new QString;
						pStr->setNum( rst );
						listNum.append( *pStr );

						tmpNum = "";	// tmpNum.setNum( rst );
						tmpCount = 0;	// tmpNum.length();
	
						tmpNum2.setNum( rst );
					}
					else
					{
						*error = ret + err + 1;
						return false;
					}

					nPosition = ret2;
				}
				else
				{
					*error = expLength;
					return false;
				}
			}

			break;
		case 8:		// else
			{
				flag = false;
			}
			break;
		}

		if ( flag )
		{
			nPosition++;
			nPreCase = nCase;
		}
	}

	if ( tmpCount != 0 )	// && nPreCase == 4 )
	{
		QString *pStr = new QString( tmpNum );
		listNum.append( *pStr );
	}


	/*
	// for debug
	for ( QStringList::Iterator iit = listNum.begin(); iit != listNum.end(); ++iit )
	{
		QString str;
		str.sprintf( "v = %s", (*iit).latin1() );
		QMessageBox::information( this, "debug", str );
	}

	for ( int ii=0 ; ii<opCount ; ii++ )
	{
		QString str;
		str.sprintf( "op = %c", listOp[ii] );
		QMessageBox::information( this, "debug", str );
	}
	*/

	
	if ( listNum.count() == 1 )
	{
		*result = listNum.first().toDouble();
		return true;
	}

	*result = 0.0;
	int count=0;
	QStringList::Iterator it = listNum.begin();
	QString op1 = *it;
	for ( ++it ; it != listNum.end() ; ++it )
	{
		QString op2 = *it;

		if ( listOp[count] == '+' )
		{
			*result = op1.toDouble() + op2.toDouble();
			op1.setNum( *result );
		}
		else if ( listOp[count] == '-' )
		{
			*result = op1.toDouble() - op2.toDouble();
			op1.setNum( *result );
		}
		count++;
	}

	if ( nPosition == expLength )
		return true;
	else
	{
		*error = nPosition;
		return false;
	}
} // parse

#define NUM_MATH_EXP_FUNC	14

// private
void CMathExpParser::initFuncVectorName()
{
	m_VectorFuncName.resize( NUM_MATH_EXP_FUNC );

	QString *pFuncName = new QString[NUM_MATH_EXP_FUNC];
	int i=0;

	pFuncName[i] = "abs";
	m_VectorFuncName.insert( i, &pFuncName[i] );
	i++;

	pFuncName[i] = "acos";
	m_VectorFuncName.insert( i, &pFuncName[i] );
	i++;

	pFuncName[i] = "asin";
	m_VectorFuncName.insert( i, &pFuncName[i] );
	i++;

	pFuncName[i] = "atan";
	m_VectorFuncName.insert( i, &pFuncName[i] );
	i++;

	pFuncName[i] = "cos";
	m_VectorFuncName.insert( i, &pFuncName[i] );
	i++;

	pFuncName[i] = "cosh";
	m_VectorFuncName.insert( i, &pFuncName[i] );
	i++;

	pFuncName[i] = "exp";
	m_VectorFuncName.insert( i, &pFuncName[i] );
	i++;

	pFuncName[i] = "log";
	m_VectorFuncName.insert( i, &pFuncName[i] );
	i++;

	pFuncName[i] = "log10";
	m_VectorFuncName.insert( i, &pFuncName[i] );
	i++;

	pFuncName[i] = "sin";
	m_VectorFuncName.insert( i, &pFuncName[i] );
	i++;

	pFuncName[i] = "sinh";
	m_VectorFuncName.insert( i, &pFuncName[i] );
	i++;

	pFuncName[i] = "tan";
	m_VectorFuncName.insert( i, &pFuncName[i] );
	i++;

	pFuncName[i] = "tanh";
	m_VectorFuncName.insert( i, &pFuncName[i] );
	i++;

	pFuncName[i] = "sqrt";
	m_VectorFuncName.insert( i, &pFuncName[i] );
	i++;
}

// private
void CMathExpParser::initFuncVector()
{
	m_VectorFunc.resize( NUM_MATH_EXP_FUNC );

	MATH_EXP_FUNC *pFunc = new MATH_EXP_FUNC[ NUM_MATH_EXP_FUNC ];
	int j=0;

	pFunc[j].func = fabs;
	m_VectorFunc.insert( j, &pFunc[j] );
	j++;

	pFunc[j].func = acos;
	m_VectorFunc.insert( j, &pFunc[j] );
	j++;

	pFunc[j].func = asin;
	m_VectorFunc.insert( j, &pFunc[j] );
	j++;

	pFunc[j].func = atan;
	m_VectorFunc.insert( j, &pFunc[j] );
	j++;

	pFunc[j].func = cos;
	m_VectorFunc.insert( j, &pFunc[j] );
	j++;

	pFunc[j].func = cosh;
	m_VectorFunc.insert( j, &pFunc[j] );
	j++;

	pFunc[j].func = exp;
	m_VectorFunc.insert( j, &pFunc[j] );
	j++;

	pFunc[j].func = log;
	m_VectorFunc.insert( j, &pFunc[j] );
	j++;

	pFunc[j].func = log10;
	m_VectorFunc.insert( j, &pFunc[j] );
	j++;

	pFunc[j].func = sin;
	m_VectorFunc.insert( j, &pFunc[j] );
	j++;

	pFunc[j].func = sinh;
	m_VectorFunc.insert( j, &pFunc[j] );
	j++;

	pFunc[j].func = tan;
	m_VectorFunc.insert( j, &pFunc[j] );
	j++;

	pFunc[j].func = tanh;
	m_VectorFunc.insert( j, &pFunc[j] );
	j++;

	pFunc[j].func = sqrt;
	m_VectorFunc.insert( j, &pFunc[j] );
	j++;
} // initFuncVector

// private
bool CMathExpParser::isOperator( QChar c )
{
	if ( c == '+' || c == '-' || c == '*' || c == '/' || c == '^' )
		return true;
	else
		return false;
} // isOperator
