#include "MontageMDIWindow.h"
#include "MontageView.h"
#include "MDIWindow.h"
#include "VolumeFile.h"

MontageMDIWindow::MontageMDIWindow( QWidget *parent, const char *name, WFlags f )
	: QMainWindow( parent, name, f )
{
	m_pVolume = NULL;
	m_pMDIWindow = NULL;
	m_pMontageView = NULL;

	// montage view
	m_pMontageView = new CMontageView( this );
	ASSERT( m_pMontageView );

	setFocusProxy( m_pMontageView );
	setCentralWidget( m_pMontageView );
} // MontageMDIWindow

MontageMDIWindow::~MontageMDIWindow()
{
	if ( m_pMDIWindow )
		m_pMDIWindow->unsetMontageView();

	if ( m_pMontageView )
	{
		delete m_pMontageView;
		m_pMontageView = NULL;
	}
} // ~MontageMDIWindow

void MontageMDIWindow::setMDIWindow( MDIWindow *pMDIWindow )
{
	m_pMDIWindow = pMDIWindow;
	ASSERT( m_pMDIWindow );
} // setMDIWindow

bool MontageMDIWindow::init( CVolume *pVolume, int nSlice )
{
	m_pVolume = pVolume;
	ASSERT( m_pVolume );
	
	m_pMontageView->init( m_pVolume, nSlice );

	QString fileName = m_pVolume->volumeFileName();
	if ( fileName.isEmpty() )
		fileName = "unnamed Montage View";

	setCaption( tr( "Montage View : " ) + fileName );

	resize( 512, 512 );	

	return true;
} // init

MDIWindow *MontageMDIWindow::mdiWindow()
{
	if ( m_pMDIWindow )
		return m_pMDIWindow;
	else
		return NULL;
} // mdiWindow

CMontageView *MontageMDIWindow::montageView()
{
	if ( m_pMontageView )
		return m_pMontageView;
	else
		return NULL;
} // montageView

CVolume *MontageMDIWindow::volume()
{
	if ( m_pVolume )
		return m_pVolume;
	else
		return NULL;
} // volume
