#include "SegmentationDlg.h"
#include "SimpleMessageDlg.h"
#include "MainWindow.h"
#include "MDIWindow.h"
#include "PlanView.h"
#include "GLMDIWindow.h"
#include "GLView.h"
#include "VolumeFile.h"
#include <qprogressbar.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qslider.h>
#include <qpushbutton.h>

#include <qmessagebox.h>	//XXX


extern CMainWindow *g_pMainWindow;


#define NUM_SHOW_PROGRESSBAR 30


CSegmentationDlg::CSegmentationDlg( QWidget *parent, const char *name, bool modal, WFlags f )
	: CSegmentationDlg_base( parent, name, modal, f )
{
	m_pOldMinThreshold = NULL;
	m_pOldMaxThreshold = NULL;
	m_pMask = NULL;

	// 1. exit button
	connect( IDC_BTN_EXIT, SIGNAL( clicked() ),
		this, SLOT( reject() ) );

	// 2. reset & reset all button
	connect( IDC_BTN_RESET, SIGNAL( clicked() ),
		this, SLOT( slotReset() ) );
	connect( IDC_BTN_RESET_ALL, SIGNAL( clicked() ),
		this, SLOT( slotResetAll() ) );

	// 3. current slice button
	connect( IDC_BTN_CURRENT_SLICE, SIGNAL( clicked() ),
		this, SLOT( slotCurrentSlice() ) );

	// 4. segment all button
	connect( IDC_BTN_SEGMENT_ALL, SIGNAL( clicked() ),
		this, SLOT( slotSegmentAll() ) );

	// 5. min threshold spinbox & slider
	connect( IDC_SPIN_MIN, SIGNAL( valueChanged( int ) ),
		IDC_SLIDER_MIN, SLOT( setValue( int ) ) );
	connect( IDC_SLIDER_MIN, SIGNAL( valueChanged( int ) ),
		IDC_SPIN_MIN, SLOT( setValue( int ) ) );

	// 6. max threshold spinbox & slider
	connect( IDC_SPIN_MAX, SIGNAL( valueChanged( int ) ),
		IDC_SLIDER_MAX, SLOT( setValue( int ) ) );
	connect( IDC_SLIDER_MAX, SIGNAL( valueChanged( int ) ),
		IDC_SPIN_MAX, SLOT( setValue( int ) ) );
} // CSegmentationDlg

CSegmentationDlg::~CSegmentationDlg()
{
	if ( m_pOldMinThreshold )
		delete [] m_pOldMinThreshold;

	if ( m_pOldMaxThreshold )
		delete [] m_pOldMaxThreshold;

	if ( m_pMask )
		delete [] m_pMask;
} // ~CSegmentationDlg

void CSegmentationDlg::init( MDIWindow *pMDIWindow )
{
	CSimpleMessageDlg *message = new CSimpleMessageDlg( g_pMainWindow, "", false );
	message->setCaption( tr( "Segmentation" ) );
	message->IDC_LBL_MESSAGE->setText( tr( "Initialize Segmentation..." ) );
//	message->exec();
	message->show();


	m_pMDIWindow = pMDIWindow;
	ASSERT( m_pMDIWindow );
	m_pMDIWindow->setSegmentationDlg( this );

	setCaption( "Segmentation : " + m_pMDIWindow->volume()->volumeFileName() );

	m_pVolume = m_pMDIWindow->volume();
	ASSERT( m_pVolume );

	m_nWidth = m_pVolume->volumeWidth();
	m_nHeight = m_pVolume->volumeHeight();
	m_nDepth = m_pVolume->volumeDepth();

	// set segmentation range
	if ( m_pVolume->isDicom() )
	{
		// if dicom, you must change segmentation range
		WORD minImageValue = m_pVolume->minImageValue();
		WORD maxImageValue = m_pVolume->maxImageValue();

		QString str;
		str.sprintf( "%d", minImageValue );
		IDC_LBL_MIN->setText( str );
		str.sprintf( "%d", maxImageValue );
		IDC_LBL_MAX->setText( str );

		IDC_SPIN_MIN->setMinValue( minImageValue );
		IDC_SPIN_MIN->setMaxValue( maxImageValue );
		IDC_SPIN_MAX->setMinValue( minImageValue );
		IDC_SPIN_MAX->setMaxValue( maxImageValue );

		IDC_SLIDER_MIN->setMinValue( minImageValue );
		IDC_SLIDER_MIN->setMaxValue( maxImageValue );
		IDC_SLIDER_MAX->setMinValue( minImageValue );
		IDC_SLIDER_MAX->setMaxValue( maxImageValue );
	}

	int nSlice = m_pMDIWindow->currentSlice();

	// set slice spin box
	IDC_SPIN_SLICE->setMinValue( 1 );
	IDC_SPIN_SLICE->setMaxValue( m_nDepth );
	IDC_SPIN_SLICE->setValue( nSlice );

	// set min & max threshold value
	m_pOldMinThreshold = new WORD[m_nDepth];
	m_pOldMaxThreshold = new WORD[m_nDepth];

	for ( WORD i=0 ; i<m_nDepth ; i++ )
	{
		m_pOldMinThreshold[i] = m_pVolume->minThreshold( i+1 );
		m_pOldMaxThreshold[i] = m_pVolume->maxThreshold( i+1 );
	}

	IDC_SPIN_MIN->setValue( m_pOldMinThreshold[nSlice-1] );
	IDC_SPIN_MAX->setValue( m_pOldMaxThreshold[nSlice-1] );

	// set mask flag
	DWORD total = m_pVolume->totalNumVoxel();
	m_pMask = new BYTE[total];
	for ( DWORD j=0 ; j<total ; j++ )
		m_pMask[j] = m_pVolume->isMaskOn( j+1 ) ? 1 : 0;

	// connect signal-slot between MDIWindow and IDC_SPIN_SLICE
	CPlanView *pPlanView = m_pMDIWindow->planView();
	connect( pPlanView, SIGNAL( sliceChanged( int) ),
		IDC_SPIN_SLICE, SLOT( setValue( int ) ) );
	connect( IDC_SPIN_SLICE, SIGNAL( valueChanged( int ) ),
		pPlanView, SLOT( slotSetSlice( int ) ) );

	// connect signal-slot between IDC_SPIN_SLICE and slotSliceChanged
	connect( IDC_SPIN_SLICE, SIGNAL( valueChanged( int ) ),
		this, SLOT( slotSliceChanged( int ) ) );

	// connect signal-slot between IDC_SPIN_MIN and slotMinChanged
	connect( IDC_SPIN_MIN, SIGNAL( valueChanged( int ) ),
		this, SLOT( slotMinChanged( int ) ) );

	// connect signal-slot between IDC_SPIN_MAX and slotMaxChanged
	connect( IDC_SPIN_MAX, SIGNAL( valueChanged( int ) ),
		this, SLOT( slotMaxChanged( int ) ) );

	if ( message )
	{
		message->hide();
		delete message;
		message = NULL;
	}
} // init

// slot
void CSegmentationDlg::slotReset()
{
	int nSlice = IDC_SPIN_SLICE->value();

	IDC_SPIN_MIN->setValue( m_pOldMinThreshold[nSlice-1] );
	IDC_SPIN_MAX->setValue( m_pOldMaxThreshold[nSlice-1] );

	int index;
	for ( WORD i=1 ; i<=m_nHeight ; i++ )
	{
		for ( WORD j=1 ; j<=m_nWidth ; j++ )
		{
			index = voxelIndex( j, i, nSlice );

			if ( m_pMask[index-1] == 1 )
				m_pVolume->setMask( index, true );
			else
				m_pVolume->setMask( index, false );
		}
	}

	m_pMDIWindow->planView()->slotSliceChanged( nSlice );
} // slotReset

// slot
void CSegmentationDlg::slotResetAll()
{
	int nSlice = IDC_SPIN_SLICE->value();

	for ( WORD i=0 ; i<m_nDepth ; i++ )
	{
		IDC_SPIN_MIN->setValue( m_pOldMinThreshold[i] );
		IDC_SPIN_MAX->setValue( m_pOldMaxThreshold[i] );
	}

	if ( m_nDepth > NUM_SHOW_PROGRESSBAR )
	{
		setMinimumSize( 510, 250 );
		setMaximumSize( 510, 250 );

		IDC_PROGRESSBAR->reset();
		IDC_PROGRESSBAR->setTotalSteps( m_nDepth );
	}

	int index;
	for ( WORD k=1 ; k<=m_nDepth ; k++ )
	{
		if ( m_nDepth > NUM_SHOW_PROGRESSBAR )
			IDC_PROGRESSBAR->setProgress( k );

		for ( WORD i=1 ; i<=m_nHeight ; i++ )
		{
			for ( WORD j=1 ; j<=m_nWidth ; j++ )
			{
				index = voxelIndex( j, i, k );

				if ( m_pMask[index-1] == 1 )
					m_pVolume->setMask( index, true );
				else
					m_pVolume->setMask( index, false );
			}
		}
	}

	if ( m_nDepth > NUM_SHOW_PROGRESSBAR )
	{
		IDC_PROGRESSBAR->reset();

		setMinimumSize( 510, 220 );
		setMaximumSize( 510, 220 );
	}

	m_pMDIWindow->planView()->slotSliceChanged( nSlice );
} // slotResetAll

// slot
void CSegmentationDlg::slotCurrentSlice()
{
	int nSlice = IDC_SPIN_SLICE->value();
	WORD wMin = IDC_SPIN_MIN->value();
	WORD wMax = IDC_SPIN_MAX->value();

	if ( wMin >= wMax )
	{
		QMessageBox::critical( this, "Error",
			tr( "Min threshold must be less than Max one" ) );
		return;
	}

	m_pVolume->setMinThreshold( nSlice, wMin );
	m_pVolume->setMaxThreshold( nSlice, wMax );

	int index;
	WORD value;
	for ( WORD i=1 ; i<=m_nHeight ; i++ )
	{
		for ( WORD j=1 ; j<=m_nWidth ; j++ )
		{
			index = voxelIndex( j, i, nSlice );
			value = m_pVolume->voxelValue( index );

			if ( value >= wMin && value <= wMax )
				m_pVolume->setMask( index, true );
			else
				m_pVolume->setMask( index, false );
		}
	}

	m_pMDIWindow->planView()->slotSliceChanged( nSlice );
} // slotCurrentSlice

// slot
void CSegmentationDlg::slotSegmentAll()
{
	int nSlice = IDC_SPIN_SLICE->value();
	WORD wMin = IDC_SPIN_MIN->value();
	WORD wMax = IDC_SPIN_MAX->value();

	if ( wMin >= wMax )
	{
		QMessageBox::critical( this, "Error",
			tr( "Min threshold must be less than Max one" ) );
		return;
	}

	for ( int i=1 ; i<=m_nDepth ; i++ )
	{
		m_pVolume->setMinThreshold( i, wMin );
		m_pVolume->setMaxThreshold( i, wMax );
	}

	if ( m_nDepth > NUM_SHOW_PROGRESSBAR )
	{
		setMinimumSize( 510, 250 );
		setMaximumSize( 510, 250 );

		IDC_PROGRESSBAR->reset();
		IDC_PROGRESSBAR->setTotalSteps( m_nDepth );
	}

	int index;
	WORD value;
	for ( WORD k=1 ; k<=m_nDepth ; k++ )
	{
		if ( m_nDepth > NUM_SHOW_PROGRESSBAR )
			IDC_PROGRESSBAR->setProgress( k );

		for ( WORD i=1 ; i<=m_nHeight ; i++ )
		{
			for ( WORD j=1 ; j<=m_nWidth ; j++ )
			{
				index = voxelIndex( j, i, k );
				value = m_pVolume->voxelValue( index );

				if ( value >= wMin && value <= wMax )
					m_pVolume->setMask( index, true );
				else
					m_pVolume->setMask( index, false );
			}
		}
	}

	if ( m_nDepth > NUM_SHOW_PROGRESSBAR )
	{
		IDC_PROGRESSBAR->reset();

		setMinimumSize( 510, 220 );
		setMaximumSize( 510, 220 );
	}

	m_pMDIWindow->planView()->slotSliceChanged( nSlice );
} // slotSegmentAll

// slot
void CSegmentationDlg::slotSliceChanged( int slice )
{
	IDC_SPIN_MIN->setValue( m_pVolume->minThreshold( slice ) );
	IDC_SPIN_MAX->setValue( m_pVolume->maxThreshold( slice ) );
} // slotSliceChanged

// slot
void CSegmentationDlg::slotMinChanged( int slice )
{
	int nMax = IDC_SPIN_MAX->value();

	if ( slice >= nMax )
		IDC_SPIN_MAX->setValue( slice );
} // slotMinChanged

// slot
void CSegmentationDlg::slotMaxChanged( int slice )
{
	int nMin = IDC_SPIN_MIN->value();

	if ( slice <= nMin )
		IDC_SPIN_MIN->setValue( slice );
} // slotMaxChanged

// protected slot
void CSegmentationDlg::reject()
{
	// if mdiwindow has opengl view
	// apply segmentation value
	bool bChanged = false;
	WORD wDepth = m_pVolume->volumeDepth();

	for ( int i=0 ; i<wDepth ; i++ )
	{
		if ( m_pOldMinThreshold[i] != m_pVolume->minThreshold( i+1 ) )
			bChanged = true;

		if ( m_pOldMaxThreshold[i] != m_pVolume->maxThreshold( i+1 ) )
			bChanged = true;
	}

	QDialog::reject();

	// segmentation is changed and if there is 3D model
	// update 3D model
	if ( bChanged && m_pMDIWindow->hasOpenGLView() )
	{
		QMessageBox::information( g_pMainWindow, "Update 3D Model",
			tr( "You have 3D Model on screen.\n\
And segmentation is changed.\n\n\
Now updating 3D Model..!!" ) );

		GLMDIWindow *opengl = m_pMDIWindow->openGLMDIWindow();
		opengl->resetSegmentation();
		opengl->openglView()->updateView();
	}

	if ( m_pMDIWindow )
		m_pMDIWindow->unsetSegmentationDlg();
} // reject

// private
DWORD CSegmentationDlg::voxelIndex( int x, int y, int slice )
{ 
	DWORD index = x + (y-1)*m_nWidth + (slice-1)*(m_nWidth*m_nHeight);
	return index;
} // voxelIndex