#ifndef _DEFINE_H_
#define _DEFINE_H_

#include "global_define.h"
#include <qstringlist.h>


#define PI 3.141592


// max number of recent files
#define MAX_RECENT_FILES	4

// voxel memory size
#define VOXEL_MEMORY_SIZE	3	// BYTE + WORD


enum
{ 
	WHITE_BLACK_COLOR = 1,
	BLACK_WHITE_COLOR,
	SINGLE_COLOR,
	INDEX_COLOR
};


// for 24bit bitmap ; don't use palette
typedef struct
{
	BYTE    rgbBlue;		// intensity of blue color
	BYTE    rgbGreen;		// intensity of green color
	BYTE    rgbRed;			// intensity of red color
} RGB;

// two-dimensional position coordinate
typedef struct
{
	WORD  x;
	WORD  y;
} POSITION_TWOD;

// three-dimensional position coordinate
typedef struct
{
	WORD  x;
	WORD  y;
	WORD  z;
} POSITION_THREED;

// rectangular region
typedef struct
{
	POSITION_TWOD  start_ele;

	WORD           width;
	WORD           height;

	WORD           depth_down;
	WORD           depth_up;
} REGION;

// keyword file structure
typedef struct
{
	QStringList  strList;
	QString      strImageFormat;

	REGION       regRegion;

	float        fVoxelWidth;
	float        fVoxelHeight;
	float        fVoxelDepth;

	bool         bWhiteIsBone;	// if true white voxel is bone, else black one is bone
} KEYWORD;

typedef struct
{
	double voxel_opacity[2];

	double mat_ambient[3];
	double mat_diffuse[3];
	double mat_specular[3];
	double mat_shinyness;

	double light_direction[3];
	double light_color[3];

	double depth_cueing[2];

	double rotate_angle[3];

	double view_port[6];
} RENDERINFO;

typedef struct
{
	float x;
	float y;
	float z;
} SRF_NODE;

typedef struct
{
	DWORD id;

	float x;
	float y;
	float z;

	DWORD count;

	// Flag for node whether the node
	// is duplicated between
	// lower mesh's Upper Nodes
	// to Upper mesh's Lower Nodes or not
	bool bDupFlag;
} FE_NODE;

typedef struct
{
	DWORD id;

	DWORD node[8];

	/*
	float fYoung;
	float fPassion;
	float fValue;
	*/

} FE_ELEMENT;

#endif // _DEFINE_H_
