/************************************************************************/
/* MGIBA: Mesh Generator for Image Based Analysis                       */
/* ==============================================                       */
/*                                                                      */
/* Written by Yongchul Kim (blksaint@freechal.com)                      */
/* http://www.diniworld.com                                             */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License.       */
/*                                                                      */
/* Please read LICENSE.txt                                              */
/************************************************************************/


#include "AnalysisVolumeDlg.h"
// #include "AnalysisVolumeThread.h"
#include "VolumeFile.h"
#include <qtextedit.h>
#include <qpushbutton.h>

CAnalysisVolumeDlg::CAnalysisVolumeDlg( QWidget *parent, const char *name, bool modal, WFlags f )
	: CAnalysisVolumeDlg_base( parent, name, modal, f )
{
	m_pVolume = NULL;

	connect( IDC_BTN_START, SIGNAL( clicked() ), this, SLOT( slotStart() ) );
	connect( IDC_BTN_STOP, SIGNAL( clicked() ), this, SLOT( slotStop() ) );
	connect( IDC_BTN_SAVE, SIGNAL( clicked() ), this, SLOT( slotSave() ) );
	connect( IDC_BTN_CLOSE, SIGNAL( clicked() ), this, SLOT( slotClose() ) );
} /* CAnalysisVolumeDlg */

CAnalysisVolumeDlg::~CAnalysisVolumeDlg()
{
} /* ~CAnalysisVolumeDlg */

void CAnalysisVolumeDlg::init( CVolume *pVolume )
{
	m_pVolume = pVolume;
	ASSERT( m_pVolume );

	// initialize thread
	m_AnalysisVolumeThread.init();		
	m_AnalysisVolumeThread.setVolume( m_pVolume );
	m_AnalysisVolumeThread.setResultWindow( IDC_TEXTEDIT_RESULTS );
	m_AnalysisVolumeThread.setResultProgressBar( IDC_PROGRESSBAR );
} /* init */

void CAnalysisVolumeDlg::start()
{
	if ( !m_AnalysisVolumeThread.running() )
	{
		IDC_TEXTEDIT_RESULTS->clear();
		m_AnalysisVolumeThread.setRunOnOff( false );
		m_AnalysisVolumeThread.start();
	}
} /* start */

void CAnalysisVolumeDlg::stop()
{
	if ( m_AnalysisVolumeThread.running() )
	{
		m_AnalysisVolumeThread.resetVolumeStruct();
		m_AnalysisVolumeThread.terminate();
	}
} /* stop */

// slot
void CAnalysisVolumeDlg::slotStart()
{
	start();
} /* slotStart */

void CAnalysisVolumeDlg::slotStop()
{
	stop();
} /* slotStop */

void CAnalysisVolumeDlg::slotSave()
{
} /* slotSave */

void CAnalysisVolumeDlg::slotClose()
{
	stop();
	QDialog::accept();
} /* slotOk */