/************************************************************************/
/* MGIBA: Mesh Generator for Image Based Analysis                       */
/* ==============================================                       */
/*                                                                      */
/* Written by Yongchul Kim (blksaint@freechal.com)                      */
/* http://www.diniworld.com                                             */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License.       */
/*                                                                      */
/* Please read LICENSE.txt                                              */
/************************************************************************/


#include "define.h"
#include "AnalysisVolumeThread.h"
#include "MainWindow.h"

#include <qdir.h>
#include <qfile.h>
#include <qdatetime.h>
#include <qtextedit.h>
#include <qprogressbar.h>
#include <qmessagebox.h>

// global
extern CMainWindow *g_pMainWindow;

void CAnalysisVolumeThread::run()
{
	if ( bRun )
	{
		sleep( 1 );
	}
	else
	{
		bRun = true;
		analysisVolume();
	}
} /* run */

void CAnalysisVolumeThread::analysisVolume()
{
	// number of accumulated feasible voxel
	// accorading to density level from 0 to 255
	DWORD dwNumFeaVoxel[256];

	// bone surface area [mm^2]
	// accorading to density level from 0 to 255
	float fBoneSurfaceArea[256];

	// bone volume [mm^3]
	// accorading to density level from 0 to 255
	float fBoneVolume[256];

	// number of bone per density from 0 to 255;
	// accorading to density level from 0 to 255
	DWORD dwNumBone[256];

	// message & time duration
	QString strMsg;
	QTime time;

	// declare variables
	POSITION_THREED posPosi;
	float fThresholdRgb;
	WORD nNumWidth, nNumHeight, nNumDepth, nNumLayer;
	DWORD /* dwTotalNumVoxel, */ dwNumVoxel, dwNumSurface[3];
	float fSurfaceArea[3];

	CQueue qBone;
	CQueue qSurrVoxel;
	CQueue qTmpVoxel;		// temporary voxel queue
	DWORD dwId1, dwId2;
	DWORD dwBoneCount, dwVoxelCount;
	DWORD i;

	DWORD dwTmp;	// temporary variable for temporary voxel file


	// get volume structure
	m_pVolume->volumeStruct( m_Volume );

	// calculate voxel surface area
	fSurfaceArea[0] = m_Volume.VolumeInfoHeader.fDx
		* m_Volume.VolumeInfoHeader.fDy;

	fSurfaceArea[1] = m_Volume.VolumeInfoHeader.fDx
		* m_Volume.VolumeInfoHeader.fDz;

	fSurfaceArea[2] = m_Volume.VolumeInfoHeader.fDy
		* m_Volume.VolumeInfoHeader.fDz;

	// set VOLUME information & number of layer
	nNumWidth = m_Volume.VolumeInfoHeader.viWidth;
	nNumHeight = m_Volume.VolumeInfoHeader.viHeight;
	nNumDepth = m_Volume.VolumeInfoHeader.viDepth;
	nNumLayer = nNumWidth * nNumHeight;

	// initialize some variables
	dwNumVoxel = 0;
	dwNumSurface[0] = 0;
	dwNumSurface[1] = 0;
	dwNumSurface[2] = 0;
	dwBoneCount = 0;

	// get total number of voxel
	m_dwTotalNumVoxel = m_pVolume->totalNumVoxel();

	// save old byte flag
	m_pByteFlag = new BYTE[m_dwTotalNumVoxel];
	if ( !m_pByteFlag )
	{
		QMessageBox::critical( g_pMainWindow, "Memory Allocation Error",
			"Memory allocation error for Temporary Flag..!!" );

		return;
	}

	// set temporary flag
	for ( i=1 ; i<=m_dwTotalNumVoxel ; i++ )
	{
		m_Volume.byteFlag[i-1] &= 0x7f;		// set 8th bit to 0
		m_pByteFlag[i-1] = m_Volume.byteFlag[i-1];
	}
	
	// write message to screen
	strMsg.sprintf( " *******[ Analysis Volume ]*******\n" );
	m_pTextEdit->append( strMsg );

	strMsg.sprintf( " %-8s  %-15s  %-15s  %-15s  %-15s",
		"Level of", "  Bone Surface ", "   Bone Volume ",
		"   Number of   ", "    Elasped    " );
	m_pTextEdit->append( strMsg );
		
	strMsg.sprintf( " %-8s  %-15s  %-15s  %-15s  %-15s\n",
		"Density", "  Area [mm^2]  ",  "     [mm^3]    ",
		"      Bone     ", "  Time [sec]   " );
	m_pTextEdit->append( strMsg );

	time.start();		// start stop watch

	for ( int j=0 ; j<=254 ; j++ )
	{
		// set progress
		m_pProgressBar->setProgress( j+1 );

		fThresholdRgb = (float)j;

		// initialize voxel byteFlag to calculate number of bone
		for ( i=1 ; i<=m_dwTotalNumVoxel ; i++ )
		{
			m_Volume.byteFlag[i-1] &= 0xbf;

			if( m_pVolume->voxelColor( i ) <= fThresholdRgb )
				m_Volume.byteFlag[i-1] |= 0x40;
		}

		for ( i=1 ; i<=m_dwTotalNumVoxel ; i++ )
		{

			/*********************************************/
			/* calculate bone m_Volume & bone surface area */
			/*********************************************/
			if ( m_pVolume->voxelColor( i ) <= fThresholdRgb )
			{
				m_pVolume->voxelPosition( i, posPosi );

				dwNumVoxel++;

				// 1st direction
				if ( posPosi.z == 1 )
					dwNumSurface[0]++;
				else
				{
					if( m_pVolume->voxelColor( i-nNumLayer ) > fThresholdRgb )
						dwNumSurface[0]++;
				}

				// 2nd direction
				if ( posPosi.z == nNumDepth )
					dwNumSurface[0]++;
				else
				{
					if ( m_pVolume->voxelColor( i+nNumLayer ) > fThresholdRgb )
						dwNumSurface[0]++;
				}

				// 3rd direction
				if ( posPosi.y == 1 )
					dwNumSurface[1]++;
				else
				{
					if( m_pVolume->voxelColor( i-nNumWidth ) > fThresholdRgb )
						dwNumSurface[1]++;
				}

				// 4th direction
				if ( posPosi.y == nNumHeight )
					dwNumSurface[1]++;
				else
				{
					if( m_pVolume->voxelColor( i+nNumWidth ) > fThresholdRgb )
						dwNumSurface[1]++;
				}

				// 5th direction
				if ( posPosi.x == nNumWidth )
					dwNumSurface[2]++;
				else
				{
					if ( m_pVolume->voxelColor( i+1 ) > fThresholdRgb )
						dwNumSurface[2]++;
				}

				// 6th direction
				if ( posPosi.x == 1 ) dwNumSurface[2]++;
				else
				{
					if( m_pVolume->voxelColor( i-1 ) > fThresholdRgb )
						dwNumSurface[2]++;
				}
			} // end if


			/****************************/
			/* calculate number of bone */
			/****************************/

			// initialize number of voxel in bone to 0
			dwVoxelCount = 0;

			if ( (m_Volume.byteFlag[i-1] & 0x40) == 0x40 )
			{
				m_Volume.byteFlag[i-1] &= 0xbf;				// initialize 7th bit to 0

				dwVoxelCount++;

				dwId1 = i;
BEGIN:
				m_pVolume->evaluateSurrVoxel( dwId1, qSurrVoxel );
			
				dwVoxelCount += qSurrVoxel.size();
			
				while ( !qSurrVoxel.is_empty() )
				{
					dwId2 = qSurrVoxel.get_front();
					qBone.insert( dwId2 );
					m_Volume.byteFlag[dwId2-1] &= 0xbf;		// initialize 7th bit to 0
				}

				if ( !qBone.is_empty() )
				{
					dwId1 = qBone.get_front();
					goto BEGIN;
				}

				dwBoneCount++;

				qTmpVoxel.insert( dwVoxelCount );
			}
		}	// for DWORD i

		// calculate bone volume
		fBoneVolume[j] = m_pVolume->voxelVolume() * dwNumVoxel;

		// calculate bone surface area
		fBoneSurfaceArea[j] = 
				  fSurfaceArea[0] * dwNumSurface[0]
				+ fSurfaceArea[1] * dwNumSurface[1]
				+ fSurfaceArea[2] * dwNumSurface[2];

		// assign number of accumulated feasible voxel
		dwNumFeaVoxel[j] = dwNumVoxel;

		// assign number of bone
		dwNumBone[j] = dwBoneCount;

		// write result to screen
		strMsg.sprintf( " %7d  %15.3f  %15.3f  %13ld  %15.3lf",
			j,
			fBoneSurfaceArea[j],
			fBoneVolume[j],
			dwNumBone[j],
			time.elapsed() / 1000.0f );
		m_pTextEdit->append( strMsg );
		
		// initialize some variables
		dwNumVoxel = 0;
		dwNumSurface[0] = 0;
		dwNumSurface[1] = 0;
		dwNumSurface[2] = 0;

		// initialize number of bone to 0
		dwBoneCount = 0;
	}	// for int j

	// for j = 255, i said before, it is special case
	{
		int nIndex = 255;

		dwBoneCount = 1;
		dwTmp = m_dwTotalNumVoxel;
		
		// calculate bone volume
		fBoneVolume[255] = m_pVolume->volumeVolume();

		// calculate bone surface area
		fBoneSurfaceArea[255] = m_pVolume->volumeSurfaceArea();

		// assign number of accumulated feasible voxel
		dwNumFeaVoxel[255] = dwTmp;

		// assign number of bone
		dwNumBone[255] = 1;

		// write result to screen
		strMsg.sprintf( " %7d  %15.3f  %15.3f  %13ld  %15.3lf",
			nIndex,
			fBoneSurfaceArea[255],
			fBoneVolume[255],
			dwNumBone[255],
			time.elapsed() / 1000.0f );
		m_pTextEdit->append( strMsg );
	}

	// set analysis results to CVolume
	for ( i=1 ; i<256 ; i++ )
	{
		m_pVolume->setNumFeaVoxel( i, dwNumFeaVoxel[i-1] );
		m_pVolume->setBoneSurfaceArea( i, fBoneSurfaceArea[i-1] );
		m_pVolume->setBoneVolume( i, fBoneVolume[i-1] );
		m_pVolume->setNumBone( i, dwNumBone[i-1] );
	}
	m_pVolume->setAnalysisOnOff( true );

	// reset volume structure
	resetVolumeStruct();

	// finish string
	m_pTextEdit->append( "=> Job Done..!!" );

	// set progress
	m_pProgressBar->setProgress( 256 );
} /* analysisVolume */

void CAnalysisVolumeThread::init()
{
	m_pVolume = NULL;
	m_pTextEdit = NULL;
	m_pProgressBar = NULL;
	
	bRun = false;
} /* init */

void CAnalysisVolumeThread::resetVolumeStruct()
{
	// initialize
	for ( DWORD i=1 ; i<=m_dwTotalNumVoxel ; i++ )
	{
		if ( ( m_Volume.byteFlag[i-1] & 0x80 ) == 0x80 )
			m_pByteFlag[i-1] |= 0x80;

		m_Volume.byteFlag[i-1] = m_pByteFlag[i-1];
	}
} /* resetVolumeStruct */

void CAnalysisVolumeThread::setRunOnOff( bool onoff )
{
	bRun = onoff;
} /* setRunOnOff */

void CAnalysisVolumeThread::setVolume( CVolume *pVolume )
{
	m_pVolume = pVolume;
	ASSERT( m_pVolume );
} /* setVolume */

void CAnalysisVolumeThread::setResultWindow( QTextEdit *pEdit )
{
	m_pTextEdit = pEdit;
	ASSERT( m_pTextEdit );
} /* setResultWindow */

void CAnalysisVolumeThread::setResultProgressBar( QProgressBar *pProgress )
{
	m_pProgressBar = pProgress;
	ASSERT( m_pProgressBar );

	m_pProgressBar->setTotalSteps( 256 );
} /* setReaultProgressBar */