/************************************************************************/
/* MGIBA: Mesh Generator for Image Based Analysis                       */
/* ==============================================                       */
/*                                                                      */
/* Written by Yongchul Kim (blksaint@freechal.com)                      */
/* http://www.diniworld.com                                             */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License.       */
/*                                                                      */
/* Please read LICENSE.txt                                              */
/************************************************************************/


#include "DicomFile.h"


CDicomFile::CDicomFile()
{
	m_bOpenFlag = false;

	m_pDicomImages = NULL;

	m_pWidth = NULL;
	m_pHeight = NULL;
	m_pFrames = NULL;
	m_pRgb = NULL;

	m_pMinValue = NULL;
	m_pMaxValue = NULL;
} /* CDicomFile */

CDicomFile::~CDicomFile()
{
	if ( m_bOpenFlag )
		close();
} /* ~CDicomFile */

void CDicomFile::close()
{
	// close opened dicom file
	dicom_close();

	// deallocating memory 
	if ( m_pDicomImages )
	{
		dicom_free( m_pDicomImages, m_nNumber );
		m_pDicomImages = NULL;
	}
	/*
	if ( m_pDicomImages )
	{
		delete m_pDicomImages;
		m_pDicomImages = NULL;
	}
	*/

	if ( m_pWidth )
	{
		delete m_pWidth;
		m_pWidth = NULL;
	}

	if ( m_pHeight )
	{
		delete m_pHeight;
		m_pHeight = NULL;
	}

	if ( m_pFrames )
	{
		delete m_pFrames;
		m_pFrames = NULL;
	}

	if ( m_pMinValue )
	{
		delete m_pMinValue;
		m_pMinValue = NULL;
	}

	if ( m_pMaxValue )
	{
		delete m_pMaxValue;
		m_pMaxValue = NULL;
	}

	m_bOpenFlag = false;
} /* close */

bool CDicomFile::init( QString fileName )
{
//	char buff[256];
	if ( dicom_read( fileName.latin1(), &m_pDicomImages, &m_nNumber, 1 ) != 0 )
		return false;

	m_pWidth = new WORD[m_nNumber];
	m_pHeight = new WORD[m_nNumber];
	m_pFrames = new WORD[m_nNumber];
	m_pRgb = new WORD[m_nNumber];
	
	m_pMinValue = new WORD[m_nNumber];
	m_pMaxValue = new WORD[m_nNumber];

	if ( !m_pWidth || !m_pHeight || !m_pFrames || !m_pRgb ||
		 !m_pMinValue || !m_pMaxValue )
		 return false;

	for ( WORD i=0 ; i<m_nNumber ; i++ )
	{
		m_pWidth[i] = m_pDicomImages[i].w;
		m_pHeight[i] = m_pDicomImages[i].h;
		m_pFrames[i] = m_pDicomImages[i].frames;
		m_pRgb[i] = m_pDicomImages[i].rgb;

		DWORD length = m_pFrames[i] * m_pWidth[i] * m_pHeight[i];
		WORD *pixel = m_pDicomImages[i].data.gray;

		WORD min = *pixel;
		WORD max = min;
		
		for ( DWORD l=length ; l ; l--, pixel++ )
		{
			if ( *pixel < min )
			{
				min = *pixel;
				continue;
			}
			if ( *pixel > max )
				max = *pixel;
		}

		m_pMinValue[i] = min;
		m_pMaxValue[i] = max;
	}

	m_bOpenFlag = true;

	return true;
} /* init */

QImage CDicomFile::image( int id /*1*/ )
{
	WORD *pix;
	BYTE *data = NULL;

	WORD width = m_pWidth[id-1];
	WORD height = m_pHeight[id-1];
	DWORD size = width*height*2;	//XXX
//	double value;
	
	data = new BYTE[size];
	memcpy( data, m_pDicomImages[id-1].data.gray, size );

	double scale = 255. / ( m_pMaxValue[id-1] - m_pMinValue[id-1] );

	QImage img( width, height, 32 );

	for ( int i=0 ; i<height ; i++ )
	{
		for ( int j=0 ; j<width ; j++ )
		{
			pix = (WORD *) data;
		//	value = (double) pix[0];
			data += 2;
			
			int color = (int) ( *pix*scale );
			img.setPixel( j, i, qRgb( color, color, color ) );
		}
	}

	/*
	if ( data )
	{
		delete data;
		data = NULL;
	}
	*/

	return img;
} /* image */

bool CDicomFile::data( BYTE *pData, int id /*1*/ )
{
	if ( !m_bOpenFlag )
		return false;

	WORD width = m_pWidth[id-1];
	WORD height = m_pHeight[id-1];
	DWORD size = width*height*2;	//XXX

	/*
	pData = new BYTE[size];
	if ( !pData )
	{
		qDebug( "cannot allocate memory for data" );
		return false;
	}
	*/

	memcpy( pData, m_pDicomImages[id-1].data.gray, size );

	return true;
} /* data */

/*
WORD CDicomFile::value( int i, int j, int id ) // id=1
{
	WORD val;

	return val;
} /* value */

WORD CDicomFile::width( int id /*1*/ )
{
	return m_pWidth[id-1];
} /* width */

WORD CDicomFile::height( int id /*1*/ )
{
	return m_pHeight[id-1];
} /* height */

WORD CDicomFile::frames( int id /*1*/ )
{
	return m_pFrames[id-1];
} /* frames */

WORD CDicomFile::rgb( int id /*1*/ )
{
	return m_pRgb[id-1];
} /* rgb */

WORD CDicomFile::minValue( int id /*1*/ )
{
	return m_pMinValue[id-1];
} /* minValue */

WORD CDicomFile::maxValue( int id /*1*/ )
{
	return m_pMaxValue[id-1];
} /* maxValue */