/************************************************************************/
/* MGIBA: Mesh Generator for Image Based Analysis                       */
/* ==============================================                       */
/*                                                                      */
/* Written by Yongchul Kim (blksaint@freechal.com)                      */
/* http://www.diniworld.com                                             */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License.       */
/*                                                                      */
/* Please read LICENSE.txt                                              */
/************************************************************************/


#include "FEModelSaveAsImageDlg.h"
#include "GLMDIWindow.h"
#include "GLView.h"
#include <qimage.h>
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qfiledialog.h>
#include <qbuttongroup.h>

CFEModelSaveAsImageDlg::CFEModelSaveAsImageDlg( QWidget *parent, const char *name, bool modal, WFlags f )
	: CFEModelSaveAsImageDlg_base( parent, name, modal, f )
{
	if ( parent->isA( "GLMDIWindow" ) )
	{
		m_pGLMDIWindow = (GLMDIWindow *) parent;
		ASSERT( m_pGLMDIWindow );
	}

	connect( IDC_BTN_FILENAME, SIGNAL( clicked() ),
		this, SLOT( slotFileName() ) );

	connect( IDC_EDIT_FILENAME, SIGNAL( textChanged ( const QString & ) ),
		this, SLOT( slotFileNameChanged( const QString & ) ) );

	connect( IDC_BTN_OK, SIGNAL( clicked() ),
		this, SLOT( slotOk() ) );

	connect( IDC_BTN_CANCEL, SIGNAL( clicked() ),
		this, SLOT( slotCancel() ) );
} /* CFEModelSaveAsImageDlg */

CFEModelSaveAsImageDlg::~CFEModelSaveAsImageDlg()
{
} /* ~CFEModelSaveAsImageDlg */

bool CFEModelSaveAsImageDlg::init( bool output )
{
	// for file type comboBox
	for ( WORD i = 0; i < QImageIO::outputFormats().count(); i++ )
	{
		QString str = QString( QImageIO::outputFormats().at( i ) );
		IDC_COMBO_FILEFORMAT->insertItem( str ); 
	}

	// for object button group
	if ( output )
	{
		/*
		if ( m_pGLMDIWindow->openglView()->isTitleShow() )
			IDC_CHECK_TITLE->setChecked( true );
		else
			IDC_CHECK_TITLE->setChecked( false );

		if ( m_pGLMDIWindow->openglView()->isBoundingBoxShow() )
			IDC_CHECK_BOUNDINGBOX->setChecked( true );
		else
			IDC_CHECK_BOUNDINGBOX->setChecked( false );

		if ( m_pGLMDIWindow->openglView()->isGridLineShow() )
			IDC_CHECK_GRIDLINE->setChecked( true );
		else
			IDC_CHECK_GRIDLINE->setChecked( false );
		*/

		// description
		IDC_LBL_DESCRIPTION->setText(
			tr(
"You can save current 3D-Model screen to image file."
			) );
	}
	else
	{
		/*
		IDC_GROUP_OBJECT->setEnabled( false );
		IDC_CHECK_TITLE->setEnabled( false );
		IDC_CHECK_BOUNDINGBOX->setEnabled( false );
		IDC_CHECK_GRIDLINE->setEnabled( false );
		*/

		// description
		IDC_LBL_DESCRIPTION->setText(
			tr(
"You can save current Render View screen to image file."
			) );
	}

	// ok button
	IDC_BTN_OK->setEnabled( false );

	return true;
} /* init */

// slot
void CFEModelSaveAsImageDlg::slotFileName()
{
	// file extension
	QString strType = IDC_COMBO_FILEFORMAT->currentText().lower();
	QString str;
	str = strType.upper() + " Image (*." + strType + ")";

	strType.insert( 0, "." );

	QString fn = QFileDialog::getSaveFileName( NULL, str, this );
    if ( !fn.isEmpty() )
	{
		// check file extension
		if ( fn.find( strType, 0 ) != -1 || fn.find( strType.upper() ) != -1 )
			;
		else
			fn.append( strType );

		IDC_EDIT_FILENAME->setText( fn );
	}
} /* slotFileName */

// slot
void CFEModelSaveAsImageDlg::slotFileNameChanged( const QString &str )
{
	if ( str.isEmpty() )
		IDC_BTN_OK->setEnabled( false );
	else
		IDC_BTN_OK->setEnabled( true );
} /* slotFileNameChanged */

// slot
void CFEModelSaveAsImageDlg::slotOk()
{
	QDialog::accept();
} /* slotOk */

// slot
void CFEModelSaveAsImageDlg::slotCancel()
{
	QDialog::reject();
} /* slotCancel */