/************************************************************************/
/* MGIBA: Mesh Generator for Image Based Analysis                       */
/* ==============================================                       */
/*                                                                      */
/* Written by Yongchul Kim (blksaint@freechal.com)                      */
/* http://www.diniworld.com                                             */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License.       */
/*                                                                      */
/* Please read LICENSE.txt                                              */
/************************************************************************/


#include "MeshFilteringDlg.h"
#include "GLMDIWindow.h"
#include "VolumeFile.h"

#include <qradiobutton.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qlabel.h>

CMeshFilteringDlg::CMeshFilteringDlg( QWidget *parent, const char *name, bool modal, WFlags f )
	: CMeshFilteringDlg_base( parent, name, modal, f )
{
	m_pGLMDIWindow = NULL;

	connect( IDC_RADIO_SURFACE, SIGNAL( stateChanged( int ) ),
		this, SLOT( slotContactStateChanged() ) );
	connect( IDC_RADIO_POINT, SIGNAL( stateChanged( int ) ),
		this, SLOT( slotContactStateChanged() ) );

	connect( IDC_RADIO_NUMBER, SIGNAL( stateChanged( int ) ),
		this, SLOT( slotFilterStateChanged() ) );
	connect( IDC_RADIO_VOLUME, SIGNAL( stateChanged( int ) ),

		this, SLOT( slotFilterStateChanged() ) );
	connect( IDC_BTN_OK, SIGNAL( clicked() ),
		this, SLOT( slotOk() ) );
	connect( IDC_BTN_CANCEL, SIGNAL( clicked() ),
		this, SLOT( slotCancel() ) );
} /* CMeshFilteringDlg */

CMeshFilteringDlg::~CMeshFilteringDlg()
{
} /* ~CMeshFilteringDlg */

bool CMeshFilteringDlg::init( GLMDIWindow *pGLMDIWindow )
{
	m_pGLMDIWindow = pGLMDIWindow;
	ASSERT( m_pGLMDIWindow );

	CVolume *pVolume = m_pGLMDIWindow->volume();
	ASSERT( pVolume );

	DWORD voxel1 = pVolume->totalNumVoxel();		// total number of voxel
	float volume1 = pVolume->volumeVolume();		// total VOLUME of volume
	DWORD voxel2 = pVolume->numCurrentFeaVoxel();	// number of current feasible voxel
	float volume2 = pVolume->feaVolume();			// VOLUME of visible volume

//	DWORD filter = pVolume->numFilterVoxel();
	DWORD filter = 0;

	QString str;

	str.sprintf( "%ld", voxel1 );
	IDC_LBL_TOTAL_VOXEL->setText( str );

	str.sprintf( "%f", volume1 );
	IDC_LBL_TOTAL_VOLUME->setText( str );

	str.sprintf( "%ld", voxel2 );
	IDC_LBL_VISIBLE_VOXEL->setText( str );

	str.sprintf( "%f", volume2 );
	IDC_LBL_VISIBLE_VOLUME->setText( str );

	str.sprintf( "%ld", filter );
	IDC_EDIT_FILTER->setText( str );

	// contact option
	if ( pVolume->isSurfaceContact() )
		IDC_RADIO_SURFACE->setChecked( true );
	else
		IDC_RADIO_POINT->setChecked( true );

	// filtering option
	IDC_RADIO_NUMBER->setChecked( true );

	return true;
} /* init */

// slot
void CMeshFilteringDlg::slotContactStateChanged()
{
	if ( IDC_RADIO_SURFACE->isOn() )
	{
		m_strMsgContact = 
			tr( "Contact : Finite Element will has Surface contact.\n\n" );
	}
	else
	{
		m_strMsgContact = 
			tr( "Contact : Finite Element will has Point contact.\n\n" );
	}

	IDC_LBL_DESCRIPTION->setText( m_strMsgContact + m_strMsgFilter );
} /* slotContactStateChanged */

// slot
void CMeshFilteringDlg::slotFilterStateChanged()
{
	if ( IDC_RADIO_NUMBER->isOn() )
	{
		IDC_LBL_FILTER->setText( tr( "Number of filtering : " ) );
		
		m_strMsgFilter = 
			tr( "Filtering : Voxel mesh which has less number of voxel than number of filtering will be filtered." );
	}
	else
	{
		IDC_LBL_FILTER->setText( tr( "Volume of filtering : " ) );

		m_strMsgFilter = 
			tr( "Filtering : Voxel mesh which has less volume than volume of filtering will be filtered." );
	}

	IDC_LBL_DESCRIPTION->setText( m_strMsgContact + m_strMsgFilter );
} /* slotRadioStateChanged */

// slot
void CMeshFilteringDlg::slotOk()
{
	QDialog::accept();
} /* slotOk */

// slot
void CMeshFilteringDlg::slotCancel()
{
	QDialog::reject();
} /* slotCancel */
