/************************************************************************/
/* MGIBA: Mesh Generator for Image Based Analysis                       */
/* ==============================================                       */
/*                                                                      */
/* Written by Yongchul Kim (blksaint@freechal.com)                      */
/* http://www.diniworld.com                                             */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License.       */
/*                                                                      */
/* Please read LICENSE.txt                                              */
/************************************************************************/


#include "MeshFilteringInfoDlg.h"
#include "MultiLineEditDlg.h"
#include "GLMDIWindow.h"
#include "GLView.h"
#include <qdir.h>
#include <qlabel.h>
#include <qpushbutton.h>

CMeshFilteringInfoDlg::CMeshFilteringInfoDlg( QWidget *parent, const char *name, bool modal, WFlags f )
	: CMeshFilteringInfoDlg_base( parent, name, modal, f )
{
	m_pGLMDIWindow = (GLMDIWindow *) parent;
	ASSERT( m_pGLMDIWindow );

	connect( IDC_BTN_OK, SIGNAL( clicked() ),
		this, SLOT( slotOk() ) );
	connect( IDC_BTN_REPORT, SIGNAL( clicked() ),
		this, SLOT( slotReport() ) );
} /* CMeshFilteringInfoDlg */

CMeshFilteringInfoDlg::~CMeshFilteringInfoDlg()
{
} /* ~CmeshFilteringInfoDlg */

bool CMeshFilteringInfoDlg::init( DWORD count, DWORD filter )
{
	QString str;

	// number of fragment
	str.sprintf( "%ld", count );
	IDC_LBL_NUMBER->setText( str );

	// number of filtered fragment
	str.sprintf( "%ld", filter );
	IDC_LBL_FILTERED->setText( str );

	// number of remained fragment
	DWORD remain = count - filter;
	str.sprintf( "%ld", remain );
	IDC_LBL_REMAINED->setText( str );

	// description
	IDC_LBL_DESCRIPTION->setText(
		tr( 
"If you want to see more detailed result, press [Report] button."
		) );

	return true;
} /* init */

// slot
void CMeshFilteringInfoDlg::slotOk()
{
	QDialog::accept();

	m_pGLMDIWindow->openglView()->updateView();
} /* slotOk */

// slot
void CMeshFilteringInfoDlg::slotReport()
{
	WFlags f = Qt::WType_TopLevel | Qt::WStyle_MinMax;
#ifdef Q_WS_WIN
	f |= Qt::WStyle_SysMenu;
#endif

	CMultiLineEditDlg *pDlg = new CMultiLineEditDlg( this, NULL, true, f );
	if ( pDlg->init( QDir::homeDirPath() + "/.mesh_filter.tmp", "Mesh Filter Report" ) )
		pDlg->show();
} /* slotReport */

// virtual
void CMeshFilteringInfoDlg::reject()
{
	QDialog::reject();

	m_pGLMDIWindow->openglView()->updateView();
} /* reject */
