/************************************************************************/
/* MGIBA: Mesh Generator for Image Based Analysis                       */
/* ==============================================                       */
/*                                                                      */
/* Written by Yongchul Kim (blksaint@freechal.com)                      */
/* http://www.diniworld.com                                             */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License.       */
/*                                                                      */
/* Please read LICENSE.txt                                              */
/************************************************************************/


#ifndef _RENDERMDIWINDOW_H_
#define _RENDERMDIWINDOW_H_

#include "global_define.h"
#include "define.h"
#include <qmainwindow.h>

#ifdef Q_WS_WIN
# include "../lib/volpack_lib/volpack.h"
#endif // Q_WS_WIN

#ifdef Q_WS_X11
# include <volpack.h>
#endif // Q_WS_X11


class CVolume;
class CRenderView;
class MDIWindow;


class RenderMDIWindow : public QMainWindow
{
	Q_OBJECT

public:
	RenderMDIWindow( QWidget *parent = 0, const char *name = 0, WFlags f = 0 );
	~RenderMDIWindow();

	bool init( CVolume *pVolume );
	
	CVolume *volume();
	void setMDIWindow( MDIWindow *pMDIWindow );
	MDIWindow *mdiWindow();

	bool setRotateX( float x );
	bool setRotateY( float y );
	bool setRotateZ( float z );

	bool zoomIn();
	bool zoomOut();
	bool plusX();
	bool minusX();
	bool plusY();
	bool minusY();
	bool plusZ();
	bool minusZ();

	BYTE *renderedImage();						// get rendered image

	int renderedImageSizeRatio() { return m_nImageSizeRatio; }

public slots:
	bool slotSaveAsImage();
	bool slotRenderedImageSize();
	bool slotRenderInfo();

private:
	bool makeRawVolume();						// make raw volume and octree
	bool renderVolume();						// render volume from raw volume
	void setDefaultRenderInfo();				// set default render information

	void deleteVPContext();

	bool renderScene();

private:
	CVolume *m_pVolume;

	CRenderView *m_pRenderView;
	MDIWindow *m_pMDIWindow;

	float m_fViewport;

	// rendering context
	vpContext *m_vpc;

	// rendering information
	RENDERINFO m_ri;

	// raw volume data
	char *m_pRawVolume;

	// rendered image
	BYTE *m_pRenderedImage;

	// rendered image size
	int m_nImageSizeRatio;

	bool m_bRotateX;
	bool m_bRotateY;
	bool m_bRotateZ;
}; // end of RenderMDIWindow

#endif // _RENDERMDIWINDOW_H_
