/************************************************************************/
/* MGIBA: Mesh Generator for Image Based Analysis                       */
/* ==============================================                       */
/*                                                                      */
/* Written by Yongchul Kim (blksaint@freechal.com)                      */
/* http://www.diniworld.com                                             */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License.       */
/*                                                                      */
/* Please read LICENSE.txt                                              */
/************************************************************************/


#ifndef _VTKQTRENDERWINDOWINTERACTOR_H_
#define _VTKQTRENDERWINDOWINTERACTOR_H_

#include "vtkQtRenderWindow.h"

#include <qtimer.h>

#include <vtkActor.h>
#include <vtkCamera.h>
#include <vtkLight.h>
#include <vtkCellPicker.h>

class vtkQtRenderWindowInteractor : public vtkQtRenderWindow
{
   Q_OBJECT

public:

   vtkQtRenderWindowInteractor( QWidget *parent = 0, const char *name = 0 );
   vtkQtRenderWindowInteractor( QGLFormat glf, QWidget *parent = 0, const char *name = 0 );

   ~vtkQtRenderWindowInteractor();

   void setLeftButtonFunction( const char* function );
   void setMiddleButtonFunction( const char* function );
   void setRightButtonFunction( const char* function );
   void setShiftLeftButtonFunction( const char* function );

   void setInteractionActor( vtkActor* actor );
   void setActorRotation( float x, float y, float z );
   void setPlane(float *ebene);
   void setInteractionTime( int time );
   void setSensitivity( float t );

   int getInteractionTime( void );
   vtkActor *getInteractionActor( void );
   float getSensitivity();
   const char *getClassName();


signals:
   /*!
      \brief This signal is emitted, whenever we have rotated an actor.
      \param xf the angle in degrees around the view-up vector of the current
      camera.
      \param yf the angle in degrees around the
      cross of the viewplane-normal and the view-up vector.
   */
   void actorRotated( float xf, float yf );

   /*!
      \brief This signal is emitted, whenever we have rolled an actor
      Rolling is rotating in the viewplane of the camera
      \param angle the angle in degrees around the viewplane-normal of
      the current camera.
   */
   void actorRolled( float angle );

   /*!
      This signal is emitted, when the interaction is finished.
      \param x the x coordinate of the pointer
      \param y the y coordinate of the pointer
   */
   void released( int x, int y );

protected:
   void mousePressEvent( QMouseEvent *event );
   void mouseReleaseEvent( QMouseEvent *event );
   void mouseMoveEvent( QMouseEvent *event );

protected slots:

   void actorRotate();
   void actorTranslate();
   void actorZoom();
   void actorRoll();
   void actorPlaneRoll();
   void actorPlaneMove();
   
   void cameraRotate();
   void cameraTranslate();
   void cameraZoom();
   void cameraRoll();
   
   void doNothing();


private:
   void vtkQtRenderWindowInteractor::DisplayToWorld( float x, float y, float z, float *worldPoint );
   void vtkQtRenderWindowInteractor::WorldToDisplay( float x, float y, float z, float *displayPoint );
   void init();

private:
   vtkCellPicker *m_pVtkCellpicker;

   vtkCamera *m_pVtkCurrentCamera;
   vtkActor *m_pVtkInteractionActor;
   vtkActor *m_pVtkCollisionActor;
   vtkLight *m_pVtkCurrentLight;

   float m_fViewUp[3];
   float m_fVpNormal[3];
   float m_fRotateY[3];
   float m_fRenCenter[3];
   float m_fViewFocus[3];
   float m_fCamPos[3];
   float m_fDeltaElevation;
   float m_fDeltaAzimuth;
   float m_fNormale[3];
   float m_fEbenekonstante;

   QTimer  *m_StepTimer;
   bool     m_bMouseEventActive;
   bool     m_bFirstCall;
   float    m_fTrackballFactor;
   float    m_fCameraRotation[3];
   int      m_nInteractionTime;
   int      m_nCurrentMousePos[2];
   int      m_nLastMousePos[2];

   char m_chLeftButtonFunction[200];
   char m_chMiddleButtonFunction[200];
   char m_chRightButtonFunction[200];
   char m_chShiftLeftButtonFunction[200];
}; // end of class vtkQtRenderWindowInteractor

#endif  // _VTKQTRENDERWINDOWINTERACTOR_H_